﻿using UnityEngine;

public class CameraControl : MonoBehaviour
{
    public float pub_DampTime = 0.2f;                   //malé časové spoždění pro pohyb kamery           
    public float pub_ScreenEdgeBuffer = 4f;             //prostor mezi hráčem a horní/dolní hranou obrazovky
    public float pub_MinSize = 8f;                      //minimální velikost kamery (maximální možné přiblížení)
    [HideInInspector] public Transform[] pub_Targets;   //pole pozic všech hráčů na scéně


    private Camera priv_Camera;                         //reference na komponentu camera      
    private float priv_ZoomSpeed;                       //rychlost přibližování kamery
    private Vector3 priv_MoveVelocity;                  //reference na proměnou třídy Vector3 se kterou pracuje metoda SmoothDamp
    private Vector3 priv_TargetPosition;               //Pozice na kterou se kamera snaží dostat

    /* metoda Awake() je volána jako první a používá se k inicializaci potřebných komponent */
    private void Awake()
    {
        //inicializace komponenty Camera
        priv_Camera = GetComponentInChildren<Camera>();
    }

    /* FixedUpdate() je načítán periodicky vždy se stejnou časovou odchylkou, používá se např. pro manipulaci s komponenty RigidBody */
    private void FixedUpdate()
    {
        Move();
        Zoom();
    }

    /* Metoda Move() přemístí kameru na určenou pozici */
    private void Move()
    {
        //volání funkce, která najde středové souřadnice mezi hráči
        CameraPosition();

        //plynulé přesunutí kamery na cílovou pozici
        transform.position = Vector3.SmoothDamp(transform.position, priv_TargetPosition, ref priv_MoveVelocity, pub_DampTime);
    }

    /* Metoda, která vyhledá středovou pozici mezi hráči a souřadnice uloží do  priv_DesiredPosition */
    private void CameraPosition()
    {
        Vector3 avPos = new Vector3();
        int numTargets = 0;

        //procházení pole ve kterém jsou uloženy souřadnice hráčů
        for (int i = 0; i < pub_Targets.Length; i++)
        {
            //Pokud je objekt aktivní provedou se výpočty
            if (pub_Targets[i].gameObject.activeSelf) {              

            //Sčítání souřadnic všech cílových objektů
            avPos += pub_Targets[i].position;

            //počet cílů
            numTargets++;
            }
        }

        //výpočet průměrné pozice
        if (numTargets > 0)
            avPos /= numTargets;

        //udržení hodnot osy Y na základní hodnotě
        avPos.y = transform.position.y;

        //naplnění proměnné cílovými souřadnicemi
        priv_TargetPosition = avPos;
    }

    /* Zoom() zajišťuje přiblížení kamery */
    private void Zoom()
    {
        //volání metody, která počítá doporučené přiblížení kamery a uložení hodnoty do proměnné
        float size = CameraSize();

        //přiblížení kamery
        priv_Camera.orthographicSize = Mathf.SmoothDamp(priv_Camera.orthographicSize, size, ref priv_ZoomSpeed, pub_DampTime);
    }

    /* metoda která počítá přiblížení kamery*/
    private float CameraSize()
    {
        //Převedení globálních souřadnic cílové pozice do lokálních hodnot kamery (výhled)
        Vector3 LocalTargetPos = transform.InverseTransformPoint(priv_TargetPosition);

        float size = 0f;

        //procházení pole ve kterém jsou uloženy souřadnice hráčů
        for (int i = 0; i < pub_Targets.Length; i++)
        {
            //Pokud je objekt aktivní, provedou se výpočty
            if (pub_Targets[i].gameObject.activeSelf) {    

            //Převedení pozic hráčů z globálních hodnot do lokálních hodnot kamery
            Vector3 LocalPLayerPos = transform.InverseTransformPoint(pub_Targets[i].position);

            //Vypočtení vzdálenosti hráče od středu kamery
            Vector3 CenterPosToTarget = LocalPLayerPos - LocalTargetPos;
            
            //Porovnání hodnot size a vzdálenosti hráče od středu kamery na vertikální ose a nahrání větší hodnoty do proměnné size
            size = Mathf.Max (size, Mathf.Abs (CenterPosToTarget.y));

            //Porovnání hodnot size a vzdálenosti hráče od středu kamery na horizontální ose a nahrání větší hodnoty do proměnné size
            size = Mathf.Max (size, Mathf.Abs (CenterPosToTarget.x) / priv_Camera.aspect);
            }
        }
        
        //zvětšení odsazení okrajů kamery od hráče
        size += pub_ScreenEdgeBuffer;

        //Ujištění že velikost kamery nebude nikdy větší než nastavené minimum
        size = Mathf.Max(size, pub_MinSize);

        return size;
    }

    /* Veřejná metoda pro počáteční nastavení kamery */
    public void SetStartPositionAndSize()
    {
        CameraPosition();

        transform.position = priv_TargetPosition;

        priv_Camera.orthographicSize = CameraSize();
    }
}