﻿using UnityEngine;
using System.Collections;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

public class GameManager : MonoBehaviour
{
    public int pub_NumRoundsToWin = 5;      //počet vítězných kol potřebných k celkové výhře
    public float pub_StartDelay = 3f;       //Délka pauzy před spuštěním kola
    public float pub_EndDelay = 3f;         //Délka pauzy na konci kola
    public CameraControl pub_CameraControl; //Reference na skript ovládající kameru
    public Text pub_MessageText;            //Reference na Testové pole v MessageCanvas
    public GameObject pub_SnowmanPrefab;    //Reference na objekt hráče uložený ve složce s assety
    public SnowmanManager[] pub_Snowmen;    //Reference na skript SnowmanManager, který provádí nastavení hráčů


    private int priv_RoundNumber;               //číslo aktuálně probíhajícího kola
    private WaitForSeconds priv_StartWait;      //Pauza před spuštěním kola
    private WaitForSeconds priv_EndWait;        //Pauza na konci kola
    private SnowmanManager priv_RoundWinner;    //hráč který vyhrál kolo
    private SnowmanManager priv_GameWinner;     //hráč který vyhrál hru


    private void Start()
    {
        //inicializace objektů časové odezvy
        priv_StartWait = new WaitForSeconds(pub_StartDelay);
        priv_EndWait = new WaitForSeconds(pub_EndDelay);

        //načtení hráčů do hry
        SpawnAllSnowmen();

        //načtení hráčů do skriptu ovládajícího kameru
        SetCameraTargets();

        //začátek herní smyčky
        StartCoroutine(GameLoop());
    }

    /* metoda která do hry načte všechny hráče */
    private void SpawnAllSnowmen()
    {
        //procházení pole hráčů
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            //Instancializace objektu hráče ze složky assetů a nastaví hráči pozici, kde se objeví
            pub_Snowmen[i].pub_Instance = Instantiate(pub_SnowmanPrefab, pub_Snowmen[i].pub_SpawnPoint.position, pub_Snowmen[i].pub_SpawnPoint.rotation) as GameObject;
            //Přidělení čísla hráči
            pub_Snowmen[i].pub_PlayerNumber = i + 1;
            //Nastavení atributů hráče ve SnowmanManageru
            pub_Snowmen[i].Setup();
        }
    }

    /* Načte objekty hráčů do skriptu, který ovládá kameru */
    private void SetCameraTargets()
    {
        //vytvoření pole pozic, kde budou později uloženy pozice hráčů
        Transform[] targets = new Transform[pub_Snowmen.Length];

        //procházení pole pozic hráčů
        for (int i = 0; i < targets.Length; i++)
        {
            //uložení pozic hráčů do pole
            targets[i] = pub_Snowmen[i].pub_Instance.transform;
        }
        //předání skriptu CameraControl pole souřadnic
        pub_CameraControl.pub_Targets = targets;
    }

    /* herní smyčka */
    private IEnumerator GameLoop()
    {
        //jednotlivé fáze kola
        yield return StartCoroutine(RoundStarting());
        yield return StartCoroutine(RoundPlaying());
        yield return StartCoroutine(RoundEnding());

        //pokud je známý výherce, tak se restartuje hra
        if (priv_GameWinner != null)
        {
            SceneManager.LoadScene(0);
        }
        else
        {
            //pokud není výherce známý načte další kolo
            StartCoroutine(GameLoop());
        }
    }

    /* počáteční fáze kola */
    private IEnumerator RoundStarting()
    {
        //vyresetování hráčů
        ResetAllSnowmen();

        //zablokování ovládání hráčů
        DisableSnowmanControl();

        //reset pozice kamery
        pub_CameraControl.SetStartPositionAndSize();

        //přičtení a zobrazení čísla probíhajícího kola
        priv_RoundNumber++;
        pub_MessageText.text = "ROUND " + priv_RoundNumber;

        //pauza a návrat do původní herní smyčky
        yield return priv_StartWait;
    }

    /* průběh kola */
    private IEnumerator RoundPlaying()
    {
        //odblokování ovládání hráčů
        EnableSnowmanControl();

        //vymazání textu z obrazovky
        pub_MessageText.text = string.Empty;

        //Dokud jsou ve hře alespoň 2 hráči, tak kolo stále probíhá
        while (!OneSnowmanLeft())
        {
            //pokud zbyde 1 a méně hráčů, program se vrátí do GameLoop() a pokračuje tam kde přestal
            yield return null;
        }      
    }

    /*konečná fáze kola*/
    private IEnumerator RoundEnding()
    {
        //zablokování ovládání
        DisableSnowmanControl();

        priv_RoundWinner = null;

        //vyhledá počet aktivních hráčů ve hře
        priv_RoundWinner = GetRoundWinner();

        //pokud instance priv_RoundWinner není prázdná, tak si přičte ve SnowmanManageru výhru
        if (priv_RoundWinner != null)
        {
            priv_RoundWinner.pub_Wins++;
        }

        //zjistí případného výherce hry
        priv_GameWinner = GetGameWinner();

        //vypsání závěrečné zprávy na obrazovku
        string message = EndMessage();
        pub_MessageText.text = message;

        //pozastavení a návrat do GameLoop()
        yield return priv_EndWait;
    }

    /* Zjištění počtu aktivních hráčů ve hře */
    private bool OneSnowmanLeft()
    {
        int numSnowmansLeft = 0;

        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            if (pub_Snowmen[i].pub_Instance.activeSelf)
                numSnowmansLeft++;
        }

        //pokud se ve hře nachází 1 a méně vrátí true, jinak false
        return numSnowmansLeft <= 1;
    }

    /* zjistí který hráč vyhrál kolo */
    private SnowmanManager GetRoundWinner()
    {
        //pokud zjistí aktivního hráče ve hře, vrátí ho jako objekt Snowmanager
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            if (pub_Snowmen[i].pub_Instance.activeSelf)
                return pub_Snowmen[i];
        }

        return null;
    }

    /* zjistí výherce hry */
    private SnowmanManager GetGameWinner()
    {
        //propočítává počet výher jednotlivých hráčů
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            //porovnává počet výher hráče a nastavené množství
            if (pub_Snowmen[i].pub_Wins == pub_NumRoundsToWin)
                return pub_Snowmen[i];
        }

        return null;
    }

    /* Zpráva při konci kola */
    private string EndMessage()
    {
        string message = "DRAW!";

        //vypíše výherce kola
        if (priv_RoundWinner != null)
            message = priv_RoundWinner.pub_ColoredPlayerText + " WINS THE ROUND!";

        message += "\n\n\n\n";

        //vypisuje počet výher jednotlivých hráčů
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            message += pub_Snowmen[i].pub_ColoredPlayerText + ": " + pub_Snowmen[i].pub_Wins + " WINS\n";
        }

        //vypíše který hráč vyhrál hru
        if (priv_GameWinner != null)
            message = priv_GameWinner.pub_ColoredPlayerText + " WINS THE GAME!";

        return message;
    }

    //vyresetování hráčů
    private void ResetAllSnowmen()
    {
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            pub_Snowmen[i].Reset();
        }
    }

    //povolení ovládání hráčů
    private void EnableSnowmanControl()
    {
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            pub_Snowmen[i].EnableControl();
        }
    }

    //zablokování ovládání hráčů
    private void DisableSnowmanControl()
    {
        for (int i = 0; i < pub_Snowmen.Length; i++)
        {
            pub_Snowmen[i].DisableControl();
        }
    }
}