﻿using System;
using UnityEngine;

[Serializable]
public class SnowmanManager
{
    public Color pub_PlayerColor;                           //Barva hráče
    public Transform pub_SpawnPoint;                        //souřadnice Spawnpointu
    [HideInInspector] public int pub_PlayerNumber;          //číslo hráče
    [HideInInspector] public string pub_ColoredPlayerText;  //obarvený text PLAYER+číslo
    [HideInInspector] public GameObject pub_Instance;       //reference na objekt hráče
    [HideInInspector] public int pub_Wins;                  //počet vyhraných kol


    private SnowmanMovement priv_Movement;          //reference na skript SnowmanMovement
    private SnowmanShooting priv_Shooting;          //reference na skript SnowmanShooting
    private SnowmanHealth priv_Health;              //reference na skript SnowmanHealth
    private GameObject priv_CanvasGameObject;       //reference na objekt obsahující životy a ukazatel nabití u hráče


    public void Setup()
    {
        //Inicializace potřebných komponent
        priv_Movement = pub_Instance.GetComponent<SnowmanMovement>();
        priv_Shooting = pub_Instance.GetComponent<SnowmanShooting>();
        priv_Health= pub_Instance.GetComponent<SnowmanHealth>();
        priv_CanvasGameObject = pub_Instance.GetComponentInChildren<Canvas>().gameObject;

        //Přidělení čísla hráče komponentám
        priv_Movement.pub_PlayerNumber = pub_PlayerNumber;
        priv_Shooting.pub_PlayerNumber = pub_PlayerNumber;

        //Obarvení životů hráče jeho barvou
        priv_Health.pub_HealtColor = pub_PlayerColor;

        //Obarvení textu Player+číslo
        pub_ColoredPlayerText = "<color=#" + ColorUtility.ToHtmlStringRGB(pub_PlayerColor) + ">PLAYER " + pub_PlayerNumber + "</color>";

    }

    //Funkce pro zablokování ovládání
    public void DisableControl()
    {
        priv_Movement.enabled = false;
        priv_Shooting.enabled = false;

        priv_CanvasGameObject.SetActive(false);
    }

    //Funkce pro odblokování ovládání
    public void EnableControl()
    {
        priv_Movement.enabled = true;
        priv_Shooting.enabled = true;

        priv_CanvasGameObject.SetActive(true);
    }

    //Reset hráče na pozici jeho spawnu
    public void Reset()
    {
        pub_Instance.transform.position = pub_SpawnPoint.position;
        pub_Instance.transform.rotation = pub_SpawnPoint.rotation;

        pub_Instance.SetActive(false);
        pub_Instance.SetActive(true);
    }
}
