﻿using UnityEngine;

public class SnowballExplosion : MonoBehaviour
{
    public LayerMask pub_SnowmanMask;               //Nastavení vrstvy ve které budou objekty ovlivněny - Layer: Players
    public ParticleSystem pub_ExplosionParticles;   //Efekt rozpadu sněhové koule při kolizi
    public AudioSource pub_ExplosionAudio;          //Zvuk sněhové koule při kolizi
    public float pub_MaxDamage = 100f;              //Máximální možné poškození
    public float pub_ExplosionForce = 1000f;        //Síla exploze projektilu
    public float pub_MaxLifeTime = 2f;              //Maximální délka životnosti projektilu
    public float pub_ExplosionRadius = 5f;          //Velikost radiu kde může být hráč poškozen

    /* metoda Start() je zavolána až po funkci Awake(), používá se k úvodnímu nastavení */
    private void Start()
    {
        Destroy(gameObject, pub_MaxLifeTime);       //Tato funkce zničí objekt po uplnutí stanovené doby
    }

    /* metoda OnTriggerEnter je spuštěná vždy když dojte ke kolizi objektu (nastavení: Is Trigger) */
    private void OnTriggerEnter(Collider other)
    {
        //Metoda OverlapSphere načte do pole colliders veškeré collidery vrstvy PLayers, které se nacházejí v určeném rádiu okolo pozice projektilu
        Collider[] colliders = Physics.OverlapSphere(transform.position, pub_ExplosionRadius, pub_SnowmanMask);

        //tento cyklus prochází pole colliders
        for (int i = 0; i < colliders.Length; i++)
        {
            //Inicializace komponenty RigidBody z položky pole colliders
            Rigidbody targetRigidbody = colliders[i].GetComponent<Rigidbody>();

            //Pokud objekt neobsahuje komponentu rigidbody, cyklus for přeskočí k dalšímu cyklu
            if (!targetRigidbody)
                continue;

            //metoda AddExplosionForce odtlačí cílový objekt rigidBody směrem od pozice projektilu v určeném radiu
            targetRigidbody.AddExplosionForce(pub_ExplosionForce, transform.position, pub_ExplosionRadius);

            //Inicializace skriptu SnowmanHealth z cílového objektu 
            SnowmanHealth targetHealth = targetRigidbody.GetComponent<SnowmanHealth>();

            //Pokud objekt neobsahuje komponentu SnowmanHealth, cyklus for přeskočí k dalšímu cyklu
            if (!targetHealth)
                continue;

            //Zavolání metody TakeDamage() umístěné ve skriptu SnowmanHealth
            targetHealth.TakeDamage(CalculateDamage(targetRigidbody.position));
        }

        //přehrání partikulárních efektů exploze a zvuku a jejich ukončení
        pub_ExplosionParticles.transform.parent = null;
        pub_ExplosionParticles.Play();
        pub_ExplosionAudio.Play();
        Destroy(pub_ExplosionParticles.gameObject, pub_ExplosionParticles.duration);

        //zničení objektu sněhové koule
        Destroy(gameObject);

    }

    /* Metoda vypočítávající poškození ze vzdálenosti projektilu a cíle */
    private float CalculateDamage(Vector3 targetPosition)
    {
        //Vypočtení vektoru mezi projektilem a cílem
        Vector3 distanceFromTarget = targetPosition - transform.position;

        //Vypočtení relativní hodnoty vzdálenosti
        float distance = (pub_ExplosionRadius - distanceFromTarget.magnitude) / pub_ExplosionRadius;

        //výpočet poškození
        float damage = distance * pub_MaxDamage;

        //Ujištění že minimální poškození bude vždy 0
        damage = Mathf.Max(0f, damage);

        return damage;
    }
}