﻿using UnityEngine;
using UnityEngine.UI;

public class SnowmanHealth : MonoBehaviour
{
    public float pub_StartingHealth = 100f;       //Počáteční životy   
    public Slider pub_Slider;                     //Reference na ukazatel životů
    public Image pub_FillImage;                   //Reference na výplň ukazatele životů
    public Color pub_HealtColor = Color.red;      //Barva životů
    public GameObject pub_ExplosionPrefab;        //Reference na kompletní animaci exploze se zvukem
    
    
    private AudioSource priv_ExplosionAudio;          //Zvuk exploze
    private ParticleSystem priv_ExplosionParticles;   //Particle efects Animace exploze
    private float priv_CurrentHealth;                 //Stávající počet životů
    private bool priv_Dead;                           //Uložení stavu živý (false), nebo mrtvý (true)

    /* metoda Awake() je volána jako první a používá se k inicializaci potřebných komponent */
    private void Awake()
    {
        priv_ExplosionParticles = Instantiate(pub_ExplosionPrefab).GetComponent<ParticleSystem>();  //Inicializace particle efects exploze
        priv_ExplosionAudio = priv_ExplosionParticles.GetComponent<AudioSource>();                  //Inicializace zvuku

        priv_ExplosionParticles.gameObject.SetActive(false);    //Deaktivace instance Particle Efektů v Hierarchy
    }

    /* metoda OnEnable() je zavolána jednou v momentě, kdy se objekt stane aktivním */
    private void OnEnable()
    {
        priv_CurrentHealth = pub_StartingHealth;    //Nastavení počtu životů na výchozí hodnotu
        priv_Dead = false;

        SetHealthUI();
    }
    
    /* Veřejná Metoda TakeDamage odečte určitý počet životů na základě vstupního parametru */
    public void TakeDamage(float amount)
    {
        priv_CurrentHealth -= amount;

        SetHealthUI();

        if (priv_CurrentHealth <= 0f && !priv_Dead)     //Podmínka ověřuje stav životů, pokud zemře přehraje explozi
        {
            OnDeath();
        }
    }

    /* Nastaví hodnotu životů do grafického ukazatele */
    private void SetHealthUI()
    {
        pub_Slider.value = priv_CurrentHealth;
        pub_FillImage.color = pub_HealtColor;
    }

    /* metoda OnDeath Slouží k přehrání exploze pokud hráč zemře */
    private void OnDeath()
    {
        priv_Dead = true;
        priv_ExplosionParticles.transform.position = transform.position;
        priv_ExplosionParticles.gameObject.SetActive(true);
        priv_ExplosionParticles.Play();
        priv_ExplosionAudio.Play();
        gameObject.SetActive(false);
    }
}