﻿using UnityEngine;
/// <summary>
/// Třída SnowmanMovement zajišťuje pohyb hráče
/// </summary>
public class SnowmanMovement : MonoBehaviour
{
    /* veřejné proměnné */
    public int pub_PlayerNumber = 1;            //číslo hráče pro získání vstupních kláves z Input Manageru
    public float pub_Speed = 12f;               //Rychlost pohybu hráče
    public float pub_TurnSpeed = 180f;          //Rychlost otáčení hráče
    
    /* privátní proměnné */   
    private string priv_MovementAxisName;       //kategorie v Input manageru pro pohyb vpřed a vzad
    private string priv_TurnAxisName;           //Kategorie v Input manageru pro otáčení vlevo a vpravo
    private Rigidbody priv_Rigidbody;           //Proměnné pro inicializaci komponenty Rigidbody 
    private float priv_MovementInputValue;      //Hodnota předávaná ze vstupních kláves pro pohyb
    private float priv_TurnInputValue;          //Hodnota předávaná ze vstupních kláves pro otáčení     

    /* metoda Awake() je volána jako první a používá se k inicializaci potřebných komponent */
    private void Awake()
    {
        priv_Rigidbody = GetComponent<Rigidbody>(); //Inicializace komponenty rigidbody
    }

    /* metoda OnEnable() je zavolána jednou v momentě, kdy se objekt stane aktivním */
    private void OnEnable ()
    {
        priv_Rigidbody.isKinematic = false;         //vypnutím isKinematic zapneme možnost ovládání hráče
        priv_MovementInputValue = 0f;               //nastavení úvodních vstupních hodnot pro pohyb
        priv_TurnInputValue = 0f;                   //nastavení úvodních vstupních hodnot pro otáčení
    }

    /* metoda OnDisable() je zavolána v případě, že objekt se stane neaktivním např. při úmrtí*/
    private void OnDisable ()
    {
        priv_Rigidbody.isKinematic = true;          //vypnutí možnosti ovládání hráče
    }

    /* metoda Start() je zavolána jednou až po funkci Awake(), používá se k úvodnímu nastavení */
    private void Start()
    {
        priv_MovementAxisName = "Vertical" + pub_PlayerNumber;  //Uložení jmena kategorie pro pohyb v Input manageru
        priv_TurnAxisName = "Horizontal" + pub_PlayerNumber;    //Uložení jmena kategorie pro otáčení v Input manageru
    }
    
    /* metoda Update() je volána při načtení každého snímku, čas načtení může být trochu rozdílný*/
    private void Update()
    {
        priv_MovementInputValue = Input.GetAxis(priv_MovementAxisName); //získání hodnoty ze vstupních kláves pro pohyb
        priv_TurnInputValue = Input.GetAxis(priv_TurnAxisName);         //získání hodnoty ze vstupních kláves pro otáčení
    }
    /* FixedUpdate() je načítán periodicky vždy se stejnou časovou odchylkou, používá se např. pro manipulaci s komponenty RigidBody */
    private void FixedUpdate()
    {
        Move();     //volání funkce pro pohyb
        Turn();     //volání funkce pro otáčení
    }


    /* Move() slouží pro pohyb vpřed a vzad */
    private void Move()
    {
        Vector3 movement = transform.forward * priv_MovementInputValue * pub_Speed * Time.deltaTime; //vypočtení vzdálenosti pohybu
        priv_Rigidbody.MovePosition(priv_Rigidbody.position + movement); //pohyb objektu na součet aktuální vzdálenosti a vypočtené vzdálenosti
    }
    /* Turn() slouží k otáčení hráče*/
    private void Turn()
    {
        float turn = priv_TurnInputValue * pub_TurnSpeed * Time.deltaTime;  //vypočtení vzdálenosti rotace

        Quaternion turnRotation = Quaternion.Euler(0f, turn,0f);            //převedení vypočtené vzdálenosti na úhel rotace

        priv_Rigidbody.MoveRotation(priv_Rigidbody.rotation * turnRotation);//provedení změn v rotaci
    }
}