﻿using UnityEngine;
using UnityEngine.UI;

public class SnowmanShooting : MonoBehaviour
{
    public int pub_PlayerNumber = 1;            //číslo hráče pro určení ovládání v Input manageru
    public Rigidbody pub_Snowball;              //reference na sněhovou kouli
    public Transform pub_FireTransform;         //reference na objekt FireTransform u hráče
    public Slider pub_AimSlider;                //reference na ukazatel střely
    public AudioSource pub_ShootingAudio;       //reference na komponentu AudioSource
    public AudioClip pub_ChargingClip;          //reference na zvuk nabíjení
    public AudioClip pub_FireClip;              //reference na zvuk výstřelu
    public float pub_MinLaunchForce = 15f;      //minimální síla nabití
    public float pub_MaxLaunchForce = 30f;      //maximální síla nabití
    public float pub_MaxChargeTime = 0.75f;     //maximální čas nabíjení

    
    private string priv_FireButton;             //Spojení ovládání s Input managerem
    private float priv_CurrentLaunchForce;      //aktuální síla nabití
    private float priv_ChargeSpeed;             //rychlost nabíjení
    private bool priv_Fired;                    //hráč vystřelil/nevystřelil

    /* metoda OnEnable() je zavolána jednou v momentě, kdy se objekt stane aktivním */
    private void OnEnable()
    {
        priv_CurrentLaunchForce = pub_MinLaunchForce;   //nastavení minimální hodnoty nabití
        pub_AimSlider.value = pub_MinLaunchForce;       //nastavení grafické interpretace minimálního nabití
    }

    /* metoda Start() je zavolána jednou na začátku, používá se k úvodnímu nastavení */
    private void Start()
    {
        priv_FireButton = "Fire" + pub_PlayerNumber;    //Spojení ovládání konkrétního hráče s Input managerem

        priv_ChargeSpeed = (pub_MaxLaunchForce - pub_MinLaunchForce) / pub_MaxChargeTime;   //výpočet rychlosti nabíjení
    }

    /* metoda Update() je volána při načtení každého snímku, čas načtení může být trochu rozdílný*/
    private void Update()
    {
        pub_AimSlider.value = pub_MinLaunchForce;   //nastavení grafické interpretace minimálního nabití

        //řetěz podmínek zajišťující nabíjení a výstřel
        if (priv_CurrentLaunchForce >= pub_MaxLaunchForce && !priv_Fired)
        {
            //výstřel pokud aktuální hodnota nabití přesáhne maximum
            priv_CurrentLaunchForce = pub_MaxLaunchForce;
            Fire();
        }
        else if (Input.GetButtonDown(priv_FireButton))
        {
            //stlačení tlačítka výstřelu
            priv_Fired = false;
            priv_CurrentLaunchForce = pub_MinLaunchForce;

            //přehrání zvuku nabíjení
            pub_ShootingAudio.clip = pub_ChargingClip;
            pub_ShootingAudio.Play();
        }
        else if (Input.GetButton(priv_FireButton) && !priv_Fired)
        {
            //držení tlačítka výstřelu a nabíjení
            priv_CurrentLaunchForce += priv_ChargeSpeed * Time.deltaTime;

            pub_AimSlider.value = priv_CurrentLaunchForce;
        }
        else if (Input.GetButtonUp(priv_FireButton) && !priv_Fired)
        {
            //uvolnění tlačítka výstřelu a výstřel
            Fire();
        }
    }

    /* Metoda Fire() načte objekt sněhové koule z assetů a vystřelí jej */
    private void Fire()
    {
        priv_Fired = true;

        //načtení sněhové koule do instance třídy Rigidbody
        Rigidbody snowballInstance = Instantiate(pub_Snowball, pub_FireTransform.position, pub_FireTransform.rotation) as Rigidbody;

        //Výstřel pomocí pozičních hodnot objektu FireTransform
        snowballInstance.velocity = priv_CurrentLaunchForce * pub_FireTransform.forward;

        //přehrání zvuku výstřelu
        pub_ShootingAudio.clip = pub_FireClip;
        pub_ShootingAudio.Play();

        //vyresetování hodnoty nabití
        priv_CurrentLaunchForce = pub_MinLaunchForce;
    }
}